package com.example.smartgarden.ui.activities;

import android.content.Intent;
import android.os.Bundle;
import android.text.TextUtils;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.FrameLayout;
import android.widget.Toast;

import androidx.appcompat.app.AppCompatActivity;

import com.example.smartgarden.R;
import com.example.smartgarden.rest.adapter.SmartGardenAdapter;
import com.example.smartgarden.rest.constants.ApiConstants;
import com.example.smartgarden.rest.model.PostResponse;
import com.example.smartgarden.rest.model.User;
import com.example.smartgarden.rest.session.SessionManager;
import com.example.smartgarden.ui.dialogs.ServerDialog;
import com.example.smartgarden.ui.interfaces.ServerDialogClickListener;

import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;

public class LoginActivity extends AppCompatActivity implements View.OnClickListener {

    private SmartGardenAdapter mApiService;
    private SessionManager mSessionManager;

    private EditText mUsernameInput;
    private EditText mPasswordInput;
    private FrameLayout mLoadingFrame;

    private ServerDialogClickListener mServerDialogListener = new ServerDialogClickListener() {
        @Override
        public void saveButtonOnClick() {
            mApiService = new SmartGardenAdapter(getApplicationContext());
        }

        @Override
        public void cancelButtonOnClick() {

        }
    };

    /**
     * Constructor
     *
     * @param savedInstanceState saved instance state
     */
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_login);

        mApiService = new SmartGardenAdapter(this);
        mSessionManager = new SessionManager(this);

        mLoadingFrame = findViewById(R.id.loadingFrame);
        mUsernameInput = findViewById(R.id.inputUsername);
        mPasswordInput = findViewById(R.id.inputPassword);

        Button mLoginButton = (Button) findViewById(R.id.btnLogin);
        Button mRegisterButton = (Button) findViewById(R.id.btnRegister);

        mLoginButton.setOnClickListener(this);
        mRegisterButton.setOnClickListener(this);

        hideLoadingFrame();

        checkExistingSession();
    }

    /**
     * onCreateOptionsMenu method
     *
     * @param menu menu object
     * @return menu inflater
     */
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.menu_initial, menu);
        return true;
    }

    /**
     * onOptionsItemSelected method
     *
     * @param item menu item
     * @return menu if activity contains menu item
     */
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        int id = item.getItemId();
        if (id == R.id.action_info) {
            showToast(getString(R.string.message_network_before_login));
        } else if (id == R.id.action_settings) {
            ServerDialog serverDialog = new ServerDialog(this, mServerDialogListener);
            serverDialog.show();
        }
        return super.onOptionsItemSelected(item);
    }

    /**
     * onClick method
     *
     * @param v clicked view
     */
    @Override
    public void onClick(View v) {
        switch (v.getId()) {
            case R.id.btnLogin:
                onLoginButtonClicked();
                break;
            case R.id.btnRegister:
                onRegisterButtonClicked();
                break;
        }
    }

    /**
     * Post api user
     *
     * @param user user to post
     */
    private void postApiLogin(User user) {
        Call<PostResponse> loginCall = mApiService.login(user);
        loginCall.enqueue(new Callback<PostResponse>() {
            @Override
            public void onResponse(Call<PostResponse> call, Response<PostResponse> response) {
                hideLoadingFrame();

                switch (response.code()) {
                    case ApiConstants.API_CODE_OK:
                        if (response.body() != null) {
                            mSessionManager.setSessionData(response.body().getUserId(),
                                    response.body().getToken());

                            Intent menuIntent = new Intent(getApplicationContext(), MenuListActivity.class);
                            startActivity(menuIntent);
                            finish();
                        }
                        break;
                    case ApiConstants.API_CODE_UNAUTHORIZED:
                        showToast(getString(R.string.message_error_authorizing));
                        break;
                    case ApiConstants.API_CODE_INTERNAL_SERVER_ERROR:
                        showToast(getString(R.string.message_error_server));
                        break;
                }
            }

            @Override
            public void onFailure(Call<PostResponse> call, Throwable t) {
                hideLoadingFrame();
                showToast(getString(R.string.message_error_reaching_server));
            }
        });
    }

    /**
     * Callback for login button clicked
     */
    private void onLoginButtonClicked() {
        String username = mUsernameInput.getText().toString();
        String password = mPasswordInput.getText().toString();

        if (TextUtils.isEmpty(username)) {
            mUsernameInput.setError(getString(R.string.message_required_field));
            return;
        } else {
            mUsernameInput.setError(null);
        }

        if (TextUtils.isEmpty(password)) {
            mPasswordInput.setError(getString(R.string.message_required_field));
            return;
        } else {
            mPasswordInput.setError(null);
        }

        User user = new User();
        user.setUsername(username);
        user.setPassword(password);

        showLoadingFrame();
        postApiLogin(user);
    }

    /**
     * Callback for register button clicked
     */
    private void onRegisterButtonClicked() {
        Intent registerIntent = new Intent(this, RegisterActivity.class);
        startActivity(registerIntent);
        finish();
    }

    /**
     * Start menu activity if exist session
     */
    private void checkExistingSession() {
        if (mSessionManager.getUserId() != null && mSessionManager.getAuthToken() != null) {
            Intent menuIntent = new Intent(getApplicationContext(), MenuListActivity.class);
            startActivity(menuIntent);
            finish();
        }
    }

    /**
     * Show loading frame
     */
    private void showLoadingFrame() {
        mLoadingFrame.setVisibility(View.VISIBLE);
    }

    /**
     * Hide loading frame
     */
    private void hideLoadingFrame() {
        mLoadingFrame.setVisibility(View.GONE);
    }

    /**
     * Show data toast
     *
     * @param message message to show
     */
    private void showToast(String message) {
        Toast.makeText(this, message, Toast.LENGTH_SHORT).show();
    }
}
