package com.example.smartgarden.ui.activities;

import android.content.Intent;
import android.os.Bundle;
import android.text.TextUtils;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.FrameLayout;
import android.widget.Toast;

import androidx.appcompat.app.AppCompatActivity;

import com.example.smartgarden.R;
import com.example.smartgarden.rest.adapter.SmartGardenAdapter;
import com.example.smartgarden.rest.constants.ApiConstants;
import com.example.smartgarden.rest.model.PostResponse;
import com.example.smartgarden.rest.model.User;
import com.example.smartgarden.ui.dialogs.ServerDialog;
import com.example.smartgarden.ui.interfaces.ServerDialogClickListener;

import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;

public class RegisterActivity extends AppCompatActivity implements View.OnClickListener {

    private SmartGardenAdapter mApiService;

    private EditText mUsernameInput;
    private EditText mPasswordInput;
    private FrameLayout mLoadingFrame;

    private ServerDialogClickListener mServerDialogListener = new ServerDialogClickListener() {
        @Override
        public void saveButtonOnClick() {
            mApiService = new SmartGardenAdapter(getApplicationContext());
        }

        @Override
        public void cancelButtonOnClick() {

        }
    };

    /**
     * onCreate method
     *
     * @param savedInstanceState saved instance state
     */
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_register);

        mApiService = new SmartGardenAdapter(this);

        mLoadingFrame = findViewById(R.id.loadingFrame);
        mUsernameInput = findViewById(R.id.inputUsername);
        mPasswordInput = findViewById(R.id.inputPassword);

        Button mRegisterButton = (Button) findViewById(R.id.btnRegister);
        Button mCancelButton = (Button) findViewById(R.id.btnCancel);

        mRegisterButton.setOnClickListener(this);
        mCancelButton.setOnClickListener(this);

        hideLoadingFrame();
    }

    /**
     * onBackPressed method
     */
    @Override
    public void onBackPressed() {
        startLoginActivity();
    }

    /**
     * onCreateOptionsMenu method
     *
     * @param menu menu object
     * @return menu inflater
     */
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.menu_initial, menu);
        return true;
    }

    /**
     * onOptionsItemSelected method
     *
     * @param item menu item
     * @return menu if activity contains menu item
     */
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        int id = item.getItemId();
        if (id == R.id.action_info) {
            showToast(getString(R.string.message_network_before_login));
        } else if (id == R.id.action_settings) {
            ServerDialog serverDialog = new ServerDialog(this, mServerDialogListener);
            serverDialog.show();
        }
        return super.onOptionsItemSelected(item);
    }

    /**
     * onClick method
     *
     * @param v clicked view
     */
    @Override
    public void onClick(View v) {
        switch (v.getId()) {
            case R.id.btnRegister:
                onRegisterButtonClicked();
                break;
            case R.id.btnCancel:
                onCancelButtonClicked();
                break;
        }
    }

    /**
     * Post api user
     *
     * @param user user to post
     */
    private void postApiRegister(User user) {
        Call<PostResponse> registerCall = mApiService.register(user);
        registerCall.enqueue(new Callback<PostResponse>() {
            @Override
            public void onResponse(Call<PostResponse> call, Response<PostResponse> response) {
                hideLoadingFrame();

                switch (response.code()) {
                    case ApiConstants.API_CODE_CREATED:
                        if (response.body() != null) {
                            showToast(getString(R.string.message_user_registered));
                            startLoginActivity();
                        }
                        break;
                    case ApiConstants.API_CODE_CONFLICT:
                        showToast(getString(R.string.message_user_exists));
                        break;
                    case ApiConstants.API_CODE_INTERNAL_SERVER_ERROR:
                        showToast(getString(R.string.message_error_server));
                        break;
                }
            }

            @Override
            public void onFailure(Call<PostResponse> call, Throwable t) {
                hideLoadingFrame();
                showToast(getString(R.string.message_error_reaching_server));
            }
        });
    }

    /**
     * Callback for register button clicked
     */
    private void onRegisterButtonClicked() {
        String username = mUsernameInput.getText().toString();
        String password = mPasswordInput.getText().toString();

        if (TextUtils.isEmpty(username)) {
            mUsernameInput.setError(getString(R.string.message_required_field));
            return;
        } else {
            mUsernameInput.setError(null);
        }

        if (TextUtils.isEmpty(password)) {
            mPasswordInput.setError(getString(R.string.message_required_field));
            return;
        } else {
            mPasswordInput.setError(null);
        }

        User user = new User();
        user.setUsername(username);
        user.setPassword(password);

        showLoadingFrame();
        postApiRegister(user);
    }

    /**
     * Callback for cancel button clicked
     */
    private void onCancelButtonClicked() {
        startLoginActivity();
    }

    /**
     * Start login activity
     */
    private void startLoginActivity() {
        Intent loginIntent = new Intent(this, LoginActivity.class);
        startActivity(loginIntent);
        finish();
    }

    /**
     * Show loading frame
     */
    private void showLoadingFrame() {
        mLoadingFrame.setVisibility(View.VISIBLE);
    }

    /**
     * Hide loading frame
     */
    private void hideLoadingFrame() {
        mLoadingFrame.setVisibility(View.GONE);
    }

    /**
     * Show data toast
     *
     * @param message message to show
     */
    private void showToast(String message) {
        Toast.makeText(this, message, Toast.LENGTH_SHORT).show();
    }
}
