package com.example.smartgarden.ui.fragments;

import android.content.Context;
import android.os.Bundle;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.FrameLayout;
import android.widget.ImageButton;
import android.widget.Spinner;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;

import com.example.smartgarden.R;
import com.example.smartgarden.rest.adapter.SmartGardenAdapter;
import com.example.smartgarden.rest.constants.ApiConstants;
import com.example.smartgarden.rest.model.Board;
import com.example.smartgarden.rest.model.Device;
import com.example.smartgarden.rest.model.Driver;
import com.example.smartgarden.rest.model.PostResponse;
import com.example.smartgarden.rest.session.SessionManager;
import com.example.smartgarden.ui.utils.Utils;
import com.google.android.material.textfield.TextInputLayout;

import java.util.ArrayList;
import java.util.List;

import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;

public class AddBoardFragment extends Fragment implements View.OnClickListener {

    private Context mContext;

    private SmartGardenAdapter mApiService;
    private SessionManager mSessionManager;
    private FrameLayout mLoadingFrame;
    private TextInputLayout mInputIdentification;
    private TextInputLayout mInputName;
    private Spinner mDriverSpinner;

    private ArrayAdapter<Driver> mDriverAdapter;

    private List<Driver> mDriverList;

    /**
     * Constructor
     */
    public AddBoardFragment() {
        // Required empty public constructor
    }

    /**
     * Fragment new instance
     *
     * @return AddBoadFragment instance
     */
    public static AddBoardFragment newInstance() {
        return new AddBoardFragment();
    }

    /**
     * onAttach method
     *
     * @param context application context
     */
    @Override
    public void onAttach(Context context) {
        super.onAttach(context);
        mContext = context;
    }

    /**
     * onCreate method
     *
     * @param savedInstanceState saved instance state
     */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        mApiService = new SmartGardenAdapter(mContext);
        mSessionManager = new SessionManager(mContext);

        mDriverList = new ArrayList<>();
    }

    /**
     * onCreateView method
     *
     * @param inflater           layout inflater
     * @param container          viewgroup container
     * @param savedInstanceState saved instance state
     * @return inflated view
     */
    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        View view = inflater.inflate(R.layout.fragment_add_board, container, false);

        mLoadingFrame = view.findViewById(R.id.loadingFrame);
        mInputIdentification = view.findViewById(R.id.layoutIdentification);
        mInputName = view.findViewById(R.id.layoutName);
        mDriverSpinner = view.findViewById(R.id.spinnerDriver);

        Button autoIdentificator = view.findViewById(R.id.btnAutoId);
        Button saveButton = view.findViewById(R.id.btnSave);
        Button resetButton = view.findViewById(R.id.btnReset);
        ImageButton driverInfoButton = view.findViewById(R.id.btnInfoDriver);

        autoIdentificator.setOnClickListener(this);
        saveButton.setOnClickListener(this);
        resetButton.setOnClickListener(this);
        driverInfoButton.setOnClickListener(this);

        return view;
    }

    /**
     * onViewCreated method
     *
     * @param view               created view
     * @param savedInstanceState saved instance state
     */
    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);

        mDriverAdapter = new ArrayAdapter<>(mContext,
                android.R.layout.simple_list_item_1, mDriverList);
        mDriverAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        mDriverSpinner.setAdapter(mDriverAdapter);

        showLoadingFrame();
        getApiDrivers();
    }

    /**
     * onClick method
     *
     * @param v clicked view
     */
    @Override
    public void onClick(View v) {
        switch (v.getId()) {
            case R.id.btnAutoId:
                onAutoIdButtonClicked();
                break;
            case R.id.btnSave:
                onSaveButtonClicked();
                break;
            case R.id.btnReset:
                onResetButtonClicked();
                break;
            case R.id.btnInfoDriver:
                onDriverInfoButtonClicked();
                break;
        }
    }

    /**
     * Get api drivers request
     */
    private void getApiDrivers() {
        Call<List<Driver>> driversCall = mApiService.getDrivers();
        driversCall.enqueue(new Callback<List<Driver>>() {
            @Override
            public void onResponse(Call<List<Driver>> call, Response<List<Driver>> response) {
                hideLoadingFrame();

                switch (response.code()) {
                    case ApiConstants.API_CODE_OK:
                        if (response.body() != null) {
                            mDriverList = response.body();

                            mDriverAdapter.clear();
                            mDriverAdapter.addAll(mDriverList);
                            mDriverAdapter.notifyDataSetChanged();
                        }

                        showToast(getString(R.string.message_data_loaded));
                        break;
                    case ApiConstants.API_CODE_UNAUTHORIZED:
                        showToast(getString(R.string.message_error_authorizing));
                        break;
                    case ApiConstants.API_CODE_INTERNAL_SERVER_ERROR:
                        showToast(getString(R.string.message_error_server));
                        break;
                }
            }

            @Override
            public void onFailure(Call<List<Driver>> call, Throwable t) {
                hideLoadingFrame();
                showToast(getString(R.string.message_error_getting_drivers));
            }
        });
    }

    /**
     * Post api board request
     *
     * @param board board to post
     */
    private void postApiBoard(Board board) {
        Call<PostResponse> addBoardCall = mApiService.addBoard(board);
        addBoardCall.enqueue(new Callback<PostResponse>() {
            @Override
            public void onResponse(Call<PostResponse> call, Response<PostResponse> response) {
                hideLoadingFrame();

                switch (response.code()) {
                    case ApiConstants.API_CODE_CREATED:
                        showToast(getString(R.string.message_board_added));
                        clearFields();
                        break;
                    case ApiConstants.API_CODE_CONFLICT:
                        showToast(getString(R.string.message_board_duplicated));
                        break;
                    case ApiConstants.API_CODE_UNAUTHORIZED:
                        showToast(getString(R.string.message_error_authorizing));
                        break;
                    case ApiConstants.API_CODE_INTERNAL_SERVER_ERROR:
                        showToast(getString(R.string.message_error_server));
                        break;
                }
            }

            @Override
            public void onFailure(Call<PostResponse> call, Throwable t) {
                hideLoadingFrame();
                showToast(getString(R.string.message_error_adding_board));
            }
        });
    }

    /**
     * Callback for setting automatic id
     */
    private void onAutoIdButtonClicked() {
        mInputIdentification.getEditText().setText(Utils.getAutomaticId());
    }

    /**
     * Callback for save button clicked
     */
    private void onSaveButtonClicked() {
        String id = mInputIdentification.getEditText().getText().toString();
        String name = mInputName.getEditText().getText().toString();
        Driver driver = (Driver) mDriverSpinner.getSelectedItem();

        if (TextUtils.isEmpty(id)) {
            mInputIdentification.setError(getString(R.string.message_required_field));
            return;
        } else {
            mInputIdentification.setError(null);
        }

        if (TextUtils.isEmpty(name)) {
            mInputName.setError(getString(R.string.message_required_field));
            return;
        } else {
            mInputName.setError(null);
        }

        if (driver == null) {
            showToast(getString(R.string.message_driver_must_be_selected));
            return;
        }

        Board board = new Board();
        board.setId(id);
        board.setName(name);
        board.setDriverId(driver.getId());
        board.setUserId(mSessionManager.getUserId());
        board.setDevices(new ArrayList<Device>());

        showLoadingFrame();
        postApiBoard(board);
    }

    /**
     * Callback for reset button clicked
     */
    private void onResetButtonClicked() {
        clearFields();
    }

    /**
     * Callback for driver info button clicked
     */
    private void onDriverInfoButtonClicked() {
        showToast(getString(R.string.message_info_driver));
    }

    /**
     * Show loading frame
     */
    private void showLoadingFrame() {
        mLoadingFrame.setVisibility(View.VISIBLE);
    }

    /**
     * Hide loading frame
     */
    private void hideLoadingFrame() {
        mLoadingFrame.setVisibility(View.GONE);
    }

    /**
     * Show toast with message
     *
     * @param message data to show
     */
    private void showToast(String message) {
        Toast.makeText(mContext, message, Toast.LENGTH_SHORT).show();
    }

    /**
     * Clear all editable fields
     */
    private void clearFields() {
        mInputIdentification.getEditText().getText().clear();
        mInputName.getEditText().getText().clear();

        if (mDriverSpinner.getCount() > 0) {
            mDriverSpinner.setSelection(0);
        }
    }
}
