package com.example.smartgarden.ui.fragments;

import android.content.Context;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.FrameLayout;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;
import androidx.recyclerview.widget.RecyclerView;
import androidx.swiperefreshlayout.widget.SwipeRefreshLayout;

import com.example.smartgarden.R;
import com.example.smartgarden.rest.adapter.SmartGardenAdapter;
import com.example.smartgarden.rest.constants.ApiConstants;
import com.example.smartgarden.rest.model.Board;
import com.example.smartgarden.rest.model.PostResponse;
import com.example.smartgarden.rest.model.Schedule;
import com.example.smartgarden.ui.adapters.BoardAdapter;
import com.example.smartgarden.ui.interfaces.DeleteClickListener;

import java.util.ArrayList;
import java.util.List;

import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;

public class DashboardFragment extends Fragment implements SwipeRefreshLayout.OnRefreshListener {

    private Context mContext;

    private SmartGardenAdapter mApiService;
    private FrameLayout mLoadingFrame;
    private RecyclerView mRecyclerView;
    private SwipeRefreshLayout mSwipeLayout;

    private List<Board> mBoardList;
    private List<Schedule> mScheduleList;

    private DeleteClickListener mDeleteClickListener = new DeleteClickListener() {
        @Override
        public void deleteBoardOnClick(String boardId) {
            onBoardDeleteClicked(boardId);
        }

        @Override
        public void deleteDeviceOnClick(String boardId, String deviceId) {
            onDeviceDeleteClicked(boardId, deviceId);
        }

        @Override
        public void deleteScheduleOnClick(String scheduleId) {
            onScheduleDeleteClicked(scheduleId);
        }
    };

    /**
     * Constructor
     */
    public DashboardFragment() {
        // Required empty public constructor
    }

    /**
     * Fragment new instance
     *
     * @return DashboardFragment instance
     */
    public static DashboardFragment newInstance() {
        return new DashboardFragment();
    }

    /**
     * onAttach method
     *
     * @param context application context
     */
    @Override
    public void onAttach(Context context) {
        super.onAttach(context);
        mContext = context;
    }

    /**
     * onCreate method
     *
     * @param savedInstanceState saved instance state
     */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        mApiService = new SmartGardenAdapter(mContext);
        mBoardList = new ArrayList<>();
        mScheduleList = new ArrayList<>();
    }

    /**
     * onCreateView method
     *
     * @param inflater           layout inflater
     * @param container          viewgroup container
     * @param savedInstanceState saved instance state
     * @return inflated view
     */
    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        View view = inflater.inflate(R.layout.fragment_dashboard, container, false);

        mLoadingFrame = view.findViewById(R.id.loadingFrame);
        mRecyclerView = view.findViewById(R.id.boardsList);
        mSwipeLayout = view.findViewById(R.id.dashboardSwipeLayout);

        mSwipeLayout.setOnRefreshListener(this);

        return view;
    }

    /**
     * onViewCreated method
     *
     * @param view               created view
     * @param savedInstanceState saved instance state
     */
    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);

        showLoadingFrame();
        getApiBoards();
    }

    /**
     * onRefresh method for swipe layout
     */
    @Override
    public void onRefresh() {
        showLoadingFrame();
        getApiBoards();
        mSwipeLayout.setRefreshing(true);
    }

    /**
     * Get api boards request
     */
    private void getApiBoards() {
        Call<List<Board>> boardsCall = mApiService.getBoards();
        boardsCall.enqueue(new Callback<List<Board>>() {
            @Override
            public void onResponse(Call<List<Board>> call, Response<List<Board>> response) {
                switch (response.code()) {
                    case ApiConstants.API_CODE_OK:
                        if (response.body() != null) {
                            mBoardList = response.body();
                        }

                        getApiSchedules();
                        break;
                    case ApiConstants.API_CODE_UNAUTHORIZED:
                        hideLoadingFrame();
                        mSwipeLayout.setRefreshing(false);
                        showToast(getString(R.string.message_error_authorizing));
                        break;
                    case ApiConstants.API_CODE_INTERNAL_SERVER_ERROR:
                        hideLoadingFrame();
                        mSwipeLayout.setRefreshing(false);
                        showToast(getString(R.string.message_error_server));
                        break;
                }
            }

            @Override
            public void onFailure(Call<List<Board>> call, Throwable t) {
                hideLoadingFrame();
                mSwipeLayout.setRefreshing(false);
                showToast(getString(R.string.message_error_getting_boards));
            }
        });
    }

    /**
     * Get api schedules
     */
    private void getApiSchedules() {
        Call<List<Schedule>> scheduleCall = mApiService.getSchedules();
        scheduleCall.enqueue(new Callback<List<Schedule>>() {
            @Override
            public void onResponse(Call<List<Schedule>> call, Response<List<Schedule>> response) {
                hideLoadingFrame();
                mSwipeLayout.setRefreshing(false);

                switch (response.code()) {
                    case ApiConstants.API_CODE_OK:
                        if (response.body() != null) {
                            mScheduleList = response.body();
                        }

                        mRecyclerView.setAdapter(new BoardAdapter(mBoardList, mScheduleList,
                                mDeleteClickListener));

                        showToast(getString(R.string.message_data_loaded));
                        break;
                    case ApiConstants.API_CODE_UNAUTHORIZED:
                        showToast(getString(R.string.message_error_authorizing));
                        break;
                    case ApiConstants.API_CODE_INTERNAL_SERVER_ERROR:
                        showToast(getString(R.string.message_error_server));
                        break;
                }
            }

            @Override
            public void onFailure(Call<List<Schedule>> call, Throwable t) {
                hideLoadingFrame();
                mSwipeLayout.setRefreshing(false);
                showToast(getString(R.string.message_error_getting_schedules));
            }
        });
    }

    /**
     * Delete board api
     *
     * @param boardId board id to delete
     */
    private void deleteApiBoard(String boardId) {
        Call<PostResponse> deleteBoardCall = mApiService.deleteBoard(boardId);
        deleteBoardCall.enqueue(new Callback<PostResponse>() {
            @Override
            public void onResponse(Call<PostResponse> call, Response<PostResponse> response) {
                switch (response.code()) {
                    case ApiConstants.API_CODE_OK:
                        getApiBoards();
                        showToast(getString(R.string.message_board_deleted));
                        break;
                    case ApiConstants.API_CODE_UNAUTHORIZED:
                        hideLoadingFrame();
                        showToast(getString(R.string.message_error_authorizing));
                        break;
                    case ApiConstants.API_CODE_INTERNAL_SERVER_ERROR:
                        hideLoadingFrame();
                        showToast(getString(R.string.message_error_server));
                        break;
                }
            }

            @Override
            public void onFailure(Call<PostResponse> call, Throwable t) {
                hideLoadingFrame();
                showToast(getString(R.string.message_error_deleting_board));
            }
        });
    }

    /**
     * Delete device api
     *
     * @param boardId  board id from device to delete
     * @param deviceId device id to delete
     */
    private void deleteApiDevice(String boardId, String deviceId) {
        Call<PostResponse> deleteDeviceCall = mApiService.deleteDevice(boardId, deviceId);
        deleteDeviceCall.enqueue(new Callback<PostResponse>() {
            @Override
            public void onResponse(Call<PostResponse> call, Response<PostResponse> response) {
                switch (response.code()) {
                    case ApiConstants.API_CODE_OK:
                        getApiBoards();
                        showToast(getString(R.string.message_device_deleted));
                        break;
                    case ApiConstants.API_CODE_UNAUTHORIZED:
                        hideLoadingFrame();
                        showToast(getString(R.string.message_error_authorizing));
                        break;
                    case ApiConstants.API_CODE_INTERNAL_SERVER_ERROR:
                        hideLoadingFrame();
                        showToast(getString(R.string.message_error_server));
                        break;
                }
            }

            @Override
            public void onFailure(Call<PostResponse> call, Throwable t) {
                hideLoadingFrame();
                showToast(getString(R.string.message_error_deleting_device));
            }
        });
    }

    /**
     * Delete schedule api
     *
     * @param scheduleId schedule to delete
     */
    private void deleteApiSchedule(String scheduleId) {
        Call<PostResponse> deleteScheduleCall = mApiService.deleteSchedule(scheduleId);
        deleteScheduleCall.enqueue(new Callback<PostResponse>() {
            @Override
            public void onResponse(Call<PostResponse> call, Response<PostResponse> response) {
                hideLoadingFrame();

                switch (response.code()) {
                    case ApiConstants.API_CODE_OK:
                        getApiBoards();
                        showToast(getString(R.string.message_schedule_deleted));
                        break;
                    case ApiConstants.API_CODE_UNAUTHORIZED:
                        showToast(getString(R.string.message_error_authorizing));
                        break;
                    case ApiConstants.API_CODE_INTERNAL_SERVER_ERROR:
                        showToast(getString(R.string.message_error_server));
                        break;
                }
            }

            @Override
            public void onFailure(Call<PostResponse> call, Throwable t) {
                hideLoadingFrame();
                showToast(getString(R.string.message_error_deleting_schedule));
            }
        });
    }

    /**
     * Callback for board delete clicked
     *
     * @param boardId board id to delete
     */
    private void onBoardDeleteClicked(String boardId) {
        showLoadingFrame();
        deleteApiBoard(boardId);
    }

    /**
     * Callback for device delete clicked
     *
     * @param boardId  board from device to delete
     * @param deviceId device id to delete
     */
    private void onDeviceDeleteClicked(String boardId, String deviceId) {
        showLoadingFrame();
        deleteApiDevice(boardId, deviceId);
    }

    /**
     * Callback for schedule delete clicked
     *
     * @param scheduleId schedule id to delete
     */
    private void onScheduleDeleteClicked(String scheduleId) {
        showLoadingFrame();
        deleteApiSchedule(scheduleId);
    }

    /**
     * Show loading frame
     */
    private void showLoadingFrame() {
        mLoadingFrame.setVisibility(View.VISIBLE);
    }

    /**
     * Hide loading frame
     */
    private void hideLoadingFrame() {
        mLoadingFrame.setVisibility(View.GONE);
    }

    /**
     * Show toast with message
     *
     * @param message data to show
     */
    private void showToast(String message) {
        Toast.makeText(mContext, message, Toast.LENGTH_SHORT).show();
    }
}
