package com.example.smartgarden.ui.fragments;

import android.Manifest;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.FrameLayout;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;

import com.example.smartgarden.R;
import com.example.smartgarden.rest.adapter.SmartGardenAdapter;
import com.example.smartgarden.rest.constants.ApiConstants;
import com.example.smartgarden.rest.model.Board;
import com.example.smartgarden.rest.model.PostResponse;
import com.example.smartgarden.rest.model.Schedule;
import com.example.smartgarden.ui.constants.UiConstants;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.Reader;
import java.io.Writer;
import java.util.Arrays;
import java.util.List;

import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;

import static android.app.Activity.RESULT_OK;


public class ExportImportFragment extends Fragment implements View.OnClickListener {

    private Context mContext;

    private SmartGardenAdapter mApiService;
    private FrameLayout mLoadingFrame;

    private Button mExportBoardButton;
    private Button mImportBoardButton;
    private Button mExportScheduleButton;
    private Button mImportScheduleButton;

    private List<Board> mBoardList;
    private List<Schedule> mScheduleList;

    /**
     * Constructor
     */
    public ExportImportFragment() {
        // Required empty public constructor
    }

    /**
     * Fragment new instance
     *
     * @return ExportImportFragment instance
     */
    public static ExportImportFragment newInstance() {
        return new ExportImportFragment();
    }

    /**
     * onAttach method
     *
     * @param context application context
     */
    @Override
    public void onAttach(Context context) {
        super.onAttach(context);
        mContext = context;
    }

    /**
     * onCreate method
     *
     * @param savedInstanceState saved instance state
     */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        mApiService = new SmartGardenAdapter(mContext);

        isStoragePermissionGranted();
    }

    /**
     * onCreateView method
     *
     * @param inflater           layout inflater
     * @param container          viewgroup container
     * @param savedInstanceState saved instance state
     * @return inflated view
     */
    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        View view = inflater.inflate(R.layout.fragment_export_import, container, false);

        mLoadingFrame = (FrameLayout) view.findViewById(R.id.loadingFrame);

        mExportBoardButton = (Button) view.findViewById(R.id.btnExportBoard);
        mImportBoardButton = (Button) view.findViewById(R.id.btnImportBoard);
        mExportScheduleButton = (Button) view.findViewById(R.id.btnExportSchedule);
        mImportScheduleButton = (Button) view.findViewById(R.id.btnImportSchedule);

        mExportBoardButton.setOnClickListener(this);
        mImportBoardButton.setOnClickListener(this);
        mExportScheduleButton.setOnClickListener(this);
        mImportScheduleButton.setOnClickListener(this);

        return view;
    }

    /**
     * onViewCreated method
     *
     * @param view               created view
     * @param savedInstanceState saved instance state
     */
    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);

        hideLoadingFrame();
    }

    /**
     * onClick method
     *
     * @param v clicked view
     */
    @Override
    public void onClick(View v) {
        switch (v.getId()) {
            case R.id.btnExportBoard:
                onExportBoardButtonClicked();
                break;
            case R.id.btnImportBoard:
                onImportBoardButtonClicked();
                break;
            case R.id.btnExportSchedule:
                onExportScheduleButtonClicked();
                break;
            case R.id.btnImportSchedule:
                onImportScheduleButtonClicked();
                break;
        }
    }

    /**
     * Check if storage permissions granted
     */
    private void isStoragePermissionGranted() {
        if (getActivity().checkSelfPermission(Manifest.permission.WRITE_EXTERNAL_STORAGE)
                != PackageManager.PERMISSION_GRANTED) {
            requestPermissions(new String[]{Manifest.permission.WRITE_EXTERNAL_STORAGE},
                    UiConstants.REQUEST_WRITE_EXTERNAL_STORAGE_CODE);
        }
    }

    /**
     * onRequestPermissionsResult method
     *
     * @param requestCode  request permission code
     * @param permissions  permissions array
     * @param grantResults permissions results
     */
    @Override
    public void onRequestPermissionsResult(int requestCode, String[] permissions,
                                           int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        if (grantResults.length > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
            mExportBoardButton.setEnabled(true);
            mExportBoardButton.getBackground().setAlpha(255);
            mImportBoardButton.setEnabled(true);
            mImportBoardButton.getBackground().setAlpha(255);
            mExportScheduleButton.setEnabled(true);
            mExportScheduleButton.getBackground().setAlpha(255);
            mImportScheduleButton.setEnabled(true);
            mImportScheduleButton.getBackground().setAlpha(255);
        } else {
            mExportBoardButton.setEnabled(false);
            mExportBoardButton.getBackground().setAlpha(128);
            mImportBoardButton.setEnabled(false);
            mImportBoardButton.getBackground().setAlpha(128);
            mExportScheduleButton.setEnabled(false);
            mExportScheduleButton.getBackground().setAlpha(128);
            mImportScheduleButton.setEnabled(false);
            mImportScheduleButton.getBackground().setAlpha(128);
            showToast("This functionality needs access to external storage. Please allow it.");
        }
    }

    /**
     * Get api boards request
     */
    private void getApiBoards() {
        Call<List<Board>> boardsCall = mApiService.getBoards();
        boardsCall.enqueue(new Callback<List<Board>>() {
            @Override
            public void onResponse(Call<List<Board>> call, Response<List<Board>> response) {
                hideLoadingFrame();

                switch (response.code()) {
                    case ApiConstants.API_CODE_OK:
                        if (response.body() != null) {
                            mBoardList = response.body();
                            setBoardFromSystem();
                        }
                        break;
                    case ApiConstants.API_CODE_UNAUTHORIZED:
                        showToast(getString(R.string.message_error_authorizing));
                        break;
                    case ApiConstants.API_CODE_INTERNAL_SERVER_ERROR:
                        showToast(getString(R.string.message_error_server));
                        break;
                }
            }

            @Override
            public void onFailure(Call<List<Board>> call, Throwable t) {
                showToast(getString(R.string.message_error_reaching_server));
                hideLoadingFrame();
            }
        });
    }

    /**
     * Get api schedules request
     */
    private void getApiSchedules() {
        Call<List<Schedule>> scheduleCall = mApiService.getSchedules();
        scheduleCall.enqueue(new Callback<List<Schedule>>() {
            @Override
            public void onResponse(Call<List<Schedule>> call, Response<List<Schedule>> response) {
                hideLoadingFrame();

                switch (response.code()) {
                    case ApiConstants.API_CODE_OK:
                        if (response.body() != null) {
                            mScheduleList = response.body();
                            setScheduleFromSystem();
                        }
                        break;
                    case ApiConstants.API_CODE_UNAUTHORIZED:
                        showToast(getString(R.string.message_error_authorizing));
                        break;
                    case ApiConstants.API_CODE_INTERNAL_SERVER_ERROR:
                        showToast(getString(R.string.message_error_server));
                        break;
                }
            }

            @Override
            public void onFailure(Call<List<Schedule>> call, Throwable t) {
                showToast(getString(R.string.message_error_reaching_server));
                hideLoadingFrame();
            }
        });
    }

    /**
     * Post api board request
     *
     * @param board board to post
     */
    private void postApiBoards(Board board) {
        Call<PostResponse> addBoardCall = mApiService.addBoard(board);
        addBoardCall.enqueue(new Callback<PostResponse>() {
            @Override
            public void onResponse(Call<PostResponse> call, Response<PostResponse> response) {
                switch (response.code()) {
                    case ApiConstants.API_CODE_CREATED:
                        showToast(getString(R.string.message_boards_imported));
                        break;
                    case ApiConstants.API_CODE_CONFLICT:
                        showToast(getString(R.string.message_board_duplicated));
                        break;
                    case ApiConstants.API_CODE_UNAUTHORIZED:
                        showToast(getString(R.string.message_error_authorizing));
                        break;
                    case ApiConstants.API_CODE_INTERNAL_SERVER_ERROR:
                        showToast(getString(R.string.message_error_server));
                        break;
                }
            }

            @Override
            public void onFailure(Call<PostResponse> call, Throwable t) {
                showToast(getString(R.string.message_error_adding_board));
            }
        });
    }

    /**
     * Post api schedule request
     *
     * @param schedule schedule to post
     */
    private void postApiSchedule(Schedule schedule) {
        Call<PostResponse> addScheduleCall = mApiService.addSchedule(schedule);
        addScheduleCall.enqueue(new Callback<PostResponse>() {
            @Override
            public void onResponse(Call<PostResponse> call, Response<PostResponse> response) {
                switch (response.code()) {
                    case ApiConstants.API_CODE_CREATED:
                        showToast(getString(R.string.message_schedules_imported));
                        break;
                    case ApiConstants.API_CODE_CONFLICT:
                        showToast(getString(R.string.message_existing_schedule));
                        break;
                    case ApiConstants.API_CODE_UNAUTHORIZED:
                        showToast(getString(R.string.message_error_authorizing));
                        break;
                    case ApiConstants.API_CODE_INTERNAL_SERVER_ERROR:
                        showToast(getString(R.string.message_error_server));
                        break;
                }
            }

            @Override
            public void onFailure(Call<PostResponse> call, Throwable t) {
                showToast(getString(R.string.error_adding_schedule));
            }
        });
    }

    /**
     * Show folder picker for saving boards in txt
     */
    private void setBoardFromSystem() {
        Intent exportIntent = new Intent(Intent.ACTION_CREATE_DOCUMENT);
        exportIntent.addCategory(Intent.CATEGORY_OPENABLE);
        exportIntent.putExtra(Intent.EXTRA_TITLE, "boards");
        exportIntent.setType("text/plain");
        startActivityForResult(exportIntent, UiConstants.WRITE_BOARD_REQUEST_CODE);
    }

    /**
     * Show folder picker for saving schedules in txt
     */
    private void setScheduleFromSystem() {
        Intent exportIntent = new Intent(Intent.ACTION_CREATE_DOCUMENT);
        exportIntent.addCategory(Intent.CATEGORY_OPENABLE);
        exportIntent.putExtra(Intent.EXTRA_TITLE, "schedules");
        exportIntent.setType("text/plain");
        startActivityForResult(exportIntent, UiConstants.WRITE_SCHEDULE_REQUEST_CODE);
    }

    /**
     * Show folder picker for getting boards from txt
     */
    private void chooseBoardFromFileSystem() {
        Intent importIntent = new Intent(Intent.ACTION_OPEN_DOCUMENT);
        importIntent.addCategory(Intent.CATEGORY_OPENABLE);
        importIntent.setType("text/plain");
        startActivityForResult(importIntent, UiConstants.READ_BOARD_REQUEST_CODE);
    }

    /**
     * Show folder picker for getting schedules from txt
     */
    private void chooseScheduleFromFileSystem() {
        Intent importIntent = new Intent(Intent.ACTION_OPEN_DOCUMENT);
        importIntent.addCategory(Intent.CATEGORY_OPENABLE);
        importIntent.setType("text/plain");
        startActivityForResult(importIntent, UiConstants.READ_SCHEDULE_REQUEST_CODE);
    }

    /**
     * Write board from URI in txt
     *
     * @param intent result intent with uri
     */
    private void writeBoardsFile(Intent intent) {
        FileOutputStream fos;

        try {
            fos = (FileOutputStream) getActivity().getContentResolver()
                    .openOutputStream(intent.getData());

            Writer fWriter;

            try {
                fWriter = new FileWriter(fos.getFD());
                Gson gson = new GsonBuilder().setPrettyPrinting().create();
                gson.toJson(mBoardList, fWriter);
                fWriter.close();
                showToast(getString(R.string.message_boards_export_success));
            } catch (Exception e) {
                showToast(getString(R.string.message_boards_export_fail));
                e.printStackTrace();
            } finally {
                fos.getFD().sync();
                fos.close();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Read board from URI in JSON object
     *
     * @param intent result intent with uri
     */
    private void readBoardsFile(Intent intent) {
        FileInputStream fis;

        try {
            fis = (FileInputStream) getActivity().getContentResolver()
                    .openInputStream(intent.getData());

            Reader fReader;

            try {
                fReader = new FileReader(fis.getFD());
                Gson gson = new Gson();
                mBoardList = Arrays.asList(gson.fromJson(fReader, Board[].class));

                for (Board board : mBoardList) {
                    if (board.getDriverId() != null) {
                        postApiBoards(board);
                    }
                }

            } catch (Exception e) {
                e.printStackTrace();
            } finally {
                fis.getFD().sync();
                fis.close();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Write schedules from URI in txt
     *
     * @param intent result intent with uri
     */
    private void writeSchedulesFile(Intent intent) {
        FileOutputStream fos;

        try {
            fos = (FileOutputStream) getActivity().getContentResolver()
                    .openOutputStream(intent.getData());

            Writer fWriter;

            try {
                fWriter = new FileWriter(fos.getFD());
                Gson gson = new GsonBuilder().setPrettyPrinting().create();
                gson.toJson(mScheduleList, fWriter);
                fWriter.close();
                showToast(getString(R.string.message_schedules_export_success));
            } catch (Exception e) {
                showToast(getString(R.string.message_schedules_export_fail));
                e.printStackTrace();
            } finally {
                fos.getFD().sync();
                fos.close();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Read schedules from URI in JSON object
     *
     * @param intent result intent with uri
     */
    private void readSchedulesFile(Intent intent) {
        FileInputStream fis;

        try {
            fis = (FileInputStream) getActivity().getContentResolver()
                    .openInputStream(intent.getData());

            Reader fReader;

            try {
                fReader = new FileReader(fis.getFD());
                Gson gson = new Gson();
                mScheduleList = Arrays.asList(gson.fromJson(fReader, Schedule[].class));

                for (Schedule schedule : mScheduleList) {
                    if (schedule.getBoardId() != null) {
                        postApiSchedule(schedule);
                    }
                }

            } catch (Exception e) {
                e.printStackTrace();
            } finally {
                fis.getFD().sync();
                fis.close();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * onActivityResult method
     *
     * @param requestCode result code from the request
     * @param resultCode  result code from the activity
     * @param resultData  result data from the activity
     */
    @Override
    public void onActivityResult(int requestCode, int resultCode, Intent resultData) {
        if (resultCode == RESULT_OK && requestCode == UiConstants.WRITE_BOARD_REQUEST_CODE) {
            writeBoardsFile(resultData);
        } else if (resultCode == RESULT_OK && requestCode == UiConstants.READ_BOARD_REQUEST_CODE) {
            readBoardsFile(resultData);
        } else if (resultCode == RESULT_OK && requestCode == UiConstants.WRITE_SCHEDULE_REQUEST_CODE) {
            writeSchedulesFile(resultData);
        } else if (resultCode == RESULT_OK && requestCode == UiConstants.READ_SCHEDULE_REQUEST_CODE) {
            readSchedulesFile(resultData);
        }
    }

    /**
     * Callback for import board button clicked
     */
    private void onImportBoardButtonClicked() {
        chooseBoardFromFileSystem();
    }

    /**
     * Callback for export board button clicked
     */
    private void onExportBoardButtonClicked() {
        showLoadingFrame();
        getApiBoards();
    }

    /**
     * Callback for import schedule button clicked
     */
    private void onImportScheduleButtonClicked() {
        chooseScheduleFromFileSystem();
    }

    /**
     * Callback for export schedule button clicked
     */
    private void onExportScheduleButtonClicked() {
        showLoadingFrame();
        getApiSchedules();
    }

    /**
     * Show toast with message
     *
     * @param message data to show
     */
    private void showToast(String message) {
        Toast.makeText(mContext, message, Toast.LENGTH_SHORT).show();
    }

    /**
     * Show loading frame
     */
    private void showLoadingFrame() {
        mLoadingFrame.setVisibility(View.VISIBLE);
    }

    /**
     * Hide loading frame
     */
    private void hideLoadingFrame() {
        mLoadingFrame.setVisibility(View.GONE);
    }

}
