package com.example.smartgarden.ui.fragments;

import android.content.Context;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.FrameLayout;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;
import androidx.recyclerview.widget.RecyclerView;

import com.example.smartgarden.R;
import com.example.smartgarden.rest.adapter.SmartGardenAdapter;
import com.example.smartgarden.rest.constants.ApiConstants;
import com.example.smartgarden.rest.model.Log;
import com.example.smartgarden.ui.adapters.LogAdapter;

import java.util.ArrayList;
import java.util.List;

import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;

public class LogsFragment extends Fragment {

    private Context mContext;

    private SmartGardenAdapter mApiService;
    private FrameLayout mLoadingFrame;
    private List<Log> mLogList;
    private RecyclerView mRecyclerView;

    /**
     * Constructor
     */
    public LogsFragment() {
        // Required empty public constructor
    }

    /**
     * Fragment new instance
     *
     * @return LogsFragment instance
     */
    public static LogsFragment newInstance() {
        return new LogsFragment();
    }

    /**
     * onAttact method
     *
     * @param context application context
     */
    @Override
    public void onAttach(Context context) {
        super.onAttach(context);
        mContext = context;
    }

    /**
     * onCreate method
     *
     * @param savedInstanceState saved instance state
     */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        mApiService = new SmartGardenAdapter(mContext);
        mLogList = new ArrayList<>();
    }

    /**
     * onCreateView method
     *
     * @param inflater           layout inflater
     * @param container          viewgroup container
     * @param savedInstanceState saved instance state
     * @return inflated view
     */
    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        View view = inflater.inflate(R.layout.fragment_logs, container, false);

        mLoadingFrame = view.findViewById(R.id.loadingFrame);
        mRecyclerView = view.findViewById(R.id.logs_list);

        return view;
    }

    /**
     * onViewCreated method
     *
     * @param view               created view
     * @param savedInstanceState saved instance state
     */
    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);

        showLoadingFrame();
        getApiLogs();
    }

    /**
     * Get api logs request
     */
    private void getApiLogs() {
        Call<List<Log>> logsCall = mApiService.getLogs();
        logsCall.enqueue(new Callback<List<Log>>() {
            @Override
            public void onResponse(Call<List<Log>> call, Response<List<Log>> response) {
                hideLoadingFrame();

                switch (response.code()) {
                    case ApiConstants.API_CODE_OK:
                        if (response.body() != null) {
                            mLogList = response.body();
                        }

                        mRecyclerView.setAdapter(new LogAdapter(mLogList));

                        showToast(getString(R.string.message_data_loaded));
                        break;
                    case ApiConstants.API_CODE_UNAUTHORIZED:
                        showToast(getString(R.string.message_error_authorizing));
                        break;
                    case ApiConstants.API_CODE_INTERNAL_SERVER_ERROR:
                        showToast(getString(R.string.message_error_server));
                        break;
                }
            }

            @Override
            public void onFailure(Call<List<Log>> call, Throwable t) {
                hideLoadingFrame();
                showToast(getString(R.string.message_error_getting_logs));
            }
        });
    }

    /**
     * Show loading frame
     */
    private void showLoadingFrame() {
        mLoadingFrame.setVisibility(View.VISIBLE);
    }

    /**
     * Hide loading frame
     */
    private void hideLoadingFrame() {
        mLoadingFrame.setVisibility(View.GONE);
    }

    /**
     * Show toast with message
     *
     * @param message data to show
     */
    private void showToast(String message) {
        Toast.makeText(mContext, message, Toast.LENGTH_SHORT).show();
    }
}
