const {
    MongoClient
} = require('mongodb');

/***************************************************************************
 * Start MongoDB connection
 ***************************************************************************/
async function initConnection() {
    const uri = "mongodb://localhost:27017";

    const client = new MongoClient(uri, {
        useNewUrlParser: true,
        useUnifiedTopology: true,
        poolSize: 10,
    });

    return await client.connect();
}

/***************************************************************************
 * Finish MongoDB connection
 ***************************************************************************/
async function finishConnection(client) {
    return await client.close();
}

/***************************************************************************
 * Check if board id exists
 ***************************************************************************/
async function checkBoardId(db, boardId) {
    try {
        const collection = db.collection("boards");
        return await collection.countDocuments({
            _id: boardId
        });
    } catch (err) {
        throw err;
    }
}

/***************************************************************************
 * Check if device id exists
 ***************************************************************************/
async function checkDeviceId(db, deviceId) {
    try {
        const collection = db.collection("boards");
        return await collection.countDocuments({
            'devices._id': deviceId
        });
    } catch (err) {
        throw err;
    }
}

/***************************************************************************
 * Check if schedule id exists
 ***************************************************************************/
async function checkScheduleId(db, scheduleId) {
    try {
        const collection = db.collection("schedules");
        return await collection.countDocuments({
            _id: scheduleId
        });
    } catch (err) {
        throw err;
    }
}

/***************************************************************************
 * Get user
 ***************************************************************************/
async function getUser(db, userName) {
    try {
        const collection = db.collection("users");
        return await collection.findOne({
            username: userName
        });
    } catch (err) {
        throw err;
    }
}

/***************************************************************************
 * Set user
 ***************************************************************************/
async function setUser(db, user) {
    try {
        const collection = db.collection("users");
        return await collection.insertOne(user);
    } catch (err) {
        throw err;
    }
}

/***************************************************************************
 * Get drivers
 ***************************************************************************/
async function getDrivers(db) {
    try {
        const collection = db.collection("drivers");
        return await collection.find().toArray();
    } catch (err) {
        throw err;
    }
}

/***************************************************************************
 * Get driver
 ***************************************************************************/
async function getDriver(db, driverId) {
    try {
        const collection = db.collection("drivers");
        return await collection.findOne({
            _id: driverId
        });
    } catch (err) {
        throw err;
    }
}

/***************************************************************************
 * Set driver
 ***************************************************************************/
async function setDriver(db, driver) {
    try {
        const collection = db.collection("drivers");
        return await collection.insertOne(driver);
    } catch (err) {
        throw err;
    }
}

/***************************************************************************
 * Get boards
 ***************************************************************************/
async function getBoards(db, userId) {
    try {
        const collection = db.collection("boards");
        return await collection.find({
            user_id: userId
        }).toArray();
    } catch (err) {
        throw err;
    }
}

/***************************************************************************
 * Set board
 ***************************************************************************/
async function setBoard(db, board) {
    try {
        const collection = db.collection("boards");
        return await collection.insertOne(board);
    } catch (err) {
        throw err;
    }
}

/***************************************************************************
 * Delete board
 ***************************************************************************/
async function deleteBoard(db, boardId) {
    try {
        const collection = db.collection("boards");
        return await collection.deleteOne({
            _id: boardId
        });
    } catch {
        throw err;
    }
}

/***************************************************************************
 * Set device
 ***************************************************************************/
async function setDevice(db, device) {
    try {
        const collection = db.collection("boards");
        return await collection.updateOne({
            _id: device.board_id
        }, {
            $push: {
                devices: device
            }
        });
    } catch (err) {
        throw err;
    }
}

/***************************************************************************
 * Delete device
 ***************************************************************************/
async function deleteDevice(db, boardId, deviceId) {
    try {
        const collection = db.collection("boards");
        return await collection.updateOne({
            _id: boardId,
        }, {
            $pull: {
                devices: {_id: deviceId}
            }
        });
    } catch (err) {
        throw err;
    }
}

/***************************************************************************
 * Set device value
 ***************************************************************************/
async function setDeviceValue(db, deviceId, value) {
    try {
        const collection = db.collection("boards");
        return await collection.updateOne({
            'devices._id': deviceId
        }, {
            $set: {
                'devices.$.latest_value': value
            }
        });
    } catch (err) {
        throw err;
    }
}

/***************************************************************************
 * Get schedules
 ***************************************************************************/
async function getSchedules(db, userId) {
    try {
        const collection = db.collection("schedules");
        return await collection.find({
            user_id: userId
        }).toArray();
    } catch (err) {
        throw err;
    }
}

/***************************************************************************
 * Set schedule
 ***************************************************************************/
async function setSchedule(db, schedule) {
    try {
        const collection = db.collection("schedules");
        return await collection.insertOne(schedule)
    } catch (err) {
        throw err;
    }
}

/***************************************************************************
 * Delete schedule by id
 ***************************************************************************/
async function deleteScheduleById(db, scheduleId) {
    try {
        const collection = db.collection("schedules");
        return await collection.deleteOne({
            _id: scheduleId
        });
    } catch {
        throw err;
    }
}

/***************************************************************************
 * Delete schedule by board
 ***************************************************************************/
async function deleteSchedulesByBoard(db, boardId) {
    try {
        const collection = db.collection("schedules");
        return await collection.deleteMany({
            board_id: boardId
        });
    } catch {
        throw err;
    }
}

/***************************************************************************
 * Delete schedule by device
 ***************************************************************************/
async function deleteSchedulesByDevice(db, deviceId) {
    try {
        const collection = db.collection("schedules");
        return await collection.deleteMany({
                actuator_id: deviceId
            },
            {
                'parameters.sensor_id': deviceId
            });
    } catch {
        throw err;
    }
}

/***************************************************************************
 * Get logs
 ***************************************************************************/
async function getLogs(db, userId) {
    try {
        const collection = db.collection("logs");
        return await collection.find({
            user_id: userId
        }).toArray();
    } catch (err) {
        throw err;
    }
}

/***************************************************************************
 * Set log
 ***************************************************************************/
async function setLog(db, log) {
    try {
        const collection = db.collection("logs");
        return await collection.insertOne(log)
    } catch (err) {
        throw err;
    }
}

module.exports = {
    initConnection,
    finishConnection,
    checkBoardId,
    checkDeviceId,
    checkScheduleId,
    getUser,
    setUser,
    getDrivers,
    getDriver,
    setDriver,
    getBoards,
    setBoard,
    deleteBoard,
    setDevice,
    deleteDevice,
    setDeviceValue,
    getSchedules,
    setSchedule,
    deleteScheduleById,
    deleteSchedulesByBoard,
    deleteSchedulesByDevice,
    getLogs,
    setLog
}