import configparser
import json
import requests
from requests.exceptions import HTTPError
import threading
from datetime import datetime
from random import randint

# =============================================================================================
# Load configuration from config.ini
# =============================================================================================
config = configparser.ConfigParser()
config.read('config/config.ini')

# =============================================================================================
# Get user configuration from config.ini
# =============================================================================================
username = config.get('USER', 'Username', fallback="")
password = config.get('USER','Password', fallback="")

# =============================================================================================
# Get server configuration from config.ini
# =============================================================================================
serverIp = config.get('SERVER', 'Ip', fallback="127.0.0.1")
serverPort = config.get('SERVER','Port', fallback=3000)

baseUrl = 'http://' + serverIp + ':' + serverPort

token = ""

# =============================================================================================
# Login and get drivers and add default driver if any exists
# =============================================================================================
try:
    loginResponse = requests.post(baseUrl + '/login', json={'username': username, 'password': password})
    loginResponse.raise_for_status()
    jsonLoginResponse = loginResponse.json()
    token = jsonLoginResponse['token']

    driversResponse = requests.get(baseUrl + '/drivers', headers={'Authorization': token})
    driversResponse.raise_for_status()
    jsonDriversResponse = driversResponse.json()

    if len(jsonDriversResponse) == 0:
        with open('drivers/drv-rpi3.json') as json_file:
            data = json.load(json_file)
            addDriverResponse = requests.post(baseUrl + '/addDriver', json=data, headers={'Authorization': token})
            addDriverResponse.raise_for_status()
            jsonAddDriverResponse = addDriverResponse.json()
            print("Default driver added to the system!")

except HTTPError as http_err:
    print(f'HTTP error occurred: {http_err}')
except Exception as err:
    print(f'Other error occurred: {err}')

# =============================================================================================
# Functions to run every WAIT_TIME_SECONDS
# =============================================================================================
sensors = []
actuators = []

def checkDevices():
    try:
        boardsResponse = requests.get(baseUrl + '/boards', headers={'Authorization': token})
        boardsResponse.raise_for_status()
        jsonBoardsResponse = boardsResponse.json()
        print("==========================================")
        print("BOARDS AT " + str(datetime.now()))
        print("==========================================")
        print("")

        sensors.clear()
        actuators.clear()

        for board in jsonBoardsResponse:
            print(("==============================="))
            print("Board name: " + board['name'])
            print(("==============================="))
            print("")

            for device in board['devices']:
                if device['type'] == "Sensor":
                    sensors.append(device['_id'])
                elif device['type'] == "Actuator":
                    actuators.append(device['_id'])
                else:
                    print("Unknow device type")
                    
                print("Device name: " + device['name']) 
                print("Device location: " + device['port']['name'])
                print("Device latest value: " + str(device['latest_value']))
                print("")
    except HTTPError as http_err:
        print(f'HTTP error occurred: {http_err}')
    except Exception as err:
        print(f'Other error occurred: {err}')


def checkRoutines():
    try:
        routinesResponse = requests.get(baseUrl + '/schedules', headers={'Authorization': token})
        routinesResponse.raise_for_status()
        jsonRoutinesResponse = routinesResponse.json()
        print("==========================================")
        print("ROUTINES AT " + str(datetime.now()))
        print("==========================================")
        print("")
        for routine in jsonRoutinesResponse:
            print("Routine name: " + routine['name'])
            print("Routine type: " + routine['type'])
            print("Routine actuator: " + routine['actuator_name'])
            if routine['type'] == 'Date':
                print("Routine start time: " + routine['parameters']['start_time'])
                print("Routine end time: " + routine['parameters']['end_time'])
            elif routine['type'] == 'Value':
                print("Routine type: " + routine['type'])
                print("Routine sensor name: " + routine['parameters']['sensor_name'])
                print("Routine operator: " + routine['parameters']['operator'])
                print("Routine value: " + str(routine['parameters']['value']))
            else:
                print("Unknow routine")
            print("")
    except HTTPError as http_err:
        print(f'HTTP error occurred: {http_err}')
    except Exception as err:
        print(f'Other error occurred: {err}')


def setRandomValues():
    try:
        for sensorId in sensors:
            addDeviceValueResponse = requests.post(baseUrl + '/addDeviceValue', json={'device_id': sensorId, 'value': randint(0, 100)}, headers={'Authorization': token})
            addDeviceValueResponse.raise_for_status()
        for actuatorId in actuators:
            addDeviceValueResponse = requests.post(baseUrl + '/addDeviceValue', json={'device_id': actuatorId, 'value': randint(0, 1)}, headers={'Authorization': token})
            addDeviceValueResponse.raise_for_status()  
    except HTTPError as http_err:
        print(f'HTTP error occurred: {http_err}')
    except Exception as err:
        print(f'Other error occurred: {err}')


# =============================================================================================
# Start main loop
# =============================================================================================
WAIT_TIME_SECONDS = 10

try:
    ticker = threading.Event()
    while not ticker.wait(WAIT_TIME_SECONDS):
        checkDevices()
        checkRoutines()
        setRandomValues()
except KeyboardInterrupt as e:
    print(e)

